from borb.pdf.document.document import Document as BorbDocument
from borb.pdf import PDF as BorbPDF

from .page import Page


class Document(BorbDocument):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


    def add_page(self, *args, **kwargs) -> Page:
        # (Overridden method)
        # If called with no arguments, we automatically create and return
        # a new Page object.
        # By default, the page is set to Letter sized instead of A4.
        if len(args) + len(kwargs) == 0:
            page = Page()
            self.add_page(page)
            return page
        else:
            return super().add_page(*args, **kwargs)


    def save(self, filename: str) -> None:
        # (New Warbler method that makes saving PDF files easier.)
        with open(filename, 'wb') as pdf_file_handle:
            BorbPDF.dumps(pdf_file_handle, self)




from borb.pdf.canvas.geometry.rectangle import Rectangle as BorbRectangle

from .warblertypes import NumberType
from decimal import Decimal

class Rectangle(BorbRectangle):
    def __init__(
        self,
        lower_left_x: NumberType,
        lower_left_y: NumberType,
        width: NumberType,
        height: NumberType,
    ):
        lower_left_x = Decimal(lower_left_x)
        lower_left_y = Decimal(lower_left_y)
        width = Decimal(width)
        height = Decimal(height)

        super().__init__(lower_left_x=lower_left_x, lower_left_y=lower_left_y, width=width, height=height)



from borb.pdf import Page as BorbPage
from borb.pdf import Paragraph as BorbParagraph
from borb.pdf import Alignment as BorbAlignment
from borb.pdf.canvas.font.font import Font as BorbFont
from borb.pdf.canvas.layout.hyphenation.hyphenation import Hyphenation as BorbHyphenation
from borb.pdf.canvas.color.color import Color as BorbColor, HexColor as BorbHexColor
from borb.pdf import SingleColumnLayout as BorbSingleColumnLayout
from borb.pdf.page.page_size import PageSize as BorbPageSize

from .util import _normalize_text_alignment, _normalize_alignment, _normalize_color, _normalize_hyphenation, _normalize_font
from .warblertypes import NumberType, ColorType, OneNumForFourType, AligmentType, OneBoolForFourType

from decimal import Decimal
from typing import Union, Tuple
import typing
from pathlib import Path

class Page(BorbPage):
    def __init__(self, width: NumberType = Decimal(612), height: NumberType = Decimal(792), size: str = ''):
        if size != '':
            width, height = pageSizeFromName(size)

        super().__init__(width, height)

        self.default_layout_obj: BorbSingleColumnLayout = None


    def add(self,
        text: str,
        respect_newlines_in_text: bool = False,
        respect_spaces_in_text: bool = False,
        font: Union[BorbFont, str] = "Helvetica",
        font_size: NumberType = Decimal(12),
        text_alignment: AligmentType = BorbAlignment.LEFT,
        alignment: AligmentType = BorbAlignment.TOP,
        font_color: ColorType = BorbHexColor("000000"),
        border: OneBoolForFourType = False,
        border_radius: OneNumForFourType = Decimal(0),
        border_color: ColorType = BorbHexColor("000000"),
        border_width: NumberType = Decimal(1),
        padding: OneNumForFourType = Decimal(0),
        margin: typing.Optional[OneNumForFourType] = None,
        fixed_leading: typing.Optional[NumberType] = None,
        multiplied_leading: typing.Optional[NumberType] = None,
        background_color: typing.Optional[ColorType] = None,
        hyphenation: typing.Optional[Union[BorbHyphenation, str]] = None,
        ):
        # (New Warbler method)
        # Adds text in a Paragraph object to a default SingleColumnLayout
        # object that it generates if needed. Returns the default layout object.

        if self.default_layout_obj is None:
            self.default_layout_obj = BorbSingleColumnLayout(self)

        # Add text as a new Paragraph object. (The Paragraph object handles type casting.)
        self.default_layout_obj.add(Paragraph(text, respect_newlines_in_text=respect_newlines_in_text, respect_spaces_in_text=respect_spaces_in_text, font=font, font_size=font_size, text_alignment=text_alignment, alignment=alignment, font_color=font_color, border=border, border_radius=border_radius, border_color=border_color, border_width=border_width, padding=padding, margin=margin, fixed_leading=fixed_leading, multiplied_leading=multiplied_leading, background_color=background_color, hyphenation=hyphenation))

        return self.default_layout_obj  # Because Layout objects return self, this method returns the layout object.

class Paragraph(BorbParagraph):
    def __init__(
        self,
        text: str,
        respect_newlines_in_text: bool = False,
        respect_spaces_in_text: bool = False,
        font: Union[BorbFont, str, Path] = "Helvetica",
        font_size: NumberType = Decimal(12),
        text_alignment: AligmentType = BorbAlignment.LEFT,
        alignment: AligmentType = BorbAlignment.TOP,
        font_color: ColorType = BorbHexColor("000000"),
        border: OneBoolForFourType = False,
        border_radius: OneNumForFourType = Decimal(0),
        border_color: ColorType = BorbHexColor("000000"),
        border_width: NumberType = Decimal(1),
        padding: OneNumForFourType = Decimal(0),
        margin: typing.Optional[OneNumForFourType] = None,
        fixed_leading: typing.Optional[NumberType] = None,
        multiplied_leading: typing.Optional[NumberType] = None,
        background_color: typing.Optional[ColorType] = None,
        hyphenation: typing.Optional[Union[BorbHyphenation, str]] = None,
    ):
        font = _normalize_font(font)

        font_size = Decimal(font_size)
        border_width = Decimal(border_width)
        if fixed_leading is not None:
            fixed_leading = Decimal(fixed_leading)
        if multiplied_leading is not None:
            multiplied_leading = Decimal(multiplied_leading)

        text_alignment = _normalize_text_alignment(text_alignment)
        vertical_alignment, horizontal_alignment = _normalize_alignment(alignment)

        font_color = _normalize_color(font_color)
        border_color = _normalize_color(border_color)
        if background_color is not None:
            background_color = _normalize_color(background_color)

        if isinstance(border, bool):
            border_top = border_right = border_bottom = border_left = border
        else:
            border_top = border[0]
            border_right = border[1]
            border_bottom = border[2]
            border_left = border[3]

        if isinstance(border_radius, Decimal) or isinstance(border_radius, int) or isinstance(border_radius, float):
            border_radius = Decimal(border_radius)
            border_radius_top_left = border_radius
            border_radius_top_right = border_radius
            border_radius_bottom_right = border_radius
            border_radius_bottom_left = border_radius
        else:
            border_radius_top_left = Decimal(border_radius[0])
            border_radius_top_right = Decimal(border_radius[1])
            border_radius_bottom_right = Decimal(border_radius[2])
            border_radius_bottom_left = Decimal(border_radius[3])

        if isinstance(padding, Decimal) or isinstance(padding, int) or isinstance(padding, float):
            padding = Decimal(padding)
            padding_top = padding
            padding_right = padding
            padding_bottom = padding
            padding_left = padding
        else:
            padding_top = Decimal(padding[0])
            padding_right = Decimal(padding[1])
            padding_bottom = Decimal(padding[2])
            padding_left = Decimal(padding[3])

        if isinstance(margin, Decimal) or isinstance(margin, int) or isinstance(margin, float) or margin is None:
            if margin is not None:
                margin = Decimal(margin)
            margin_top = margin
            margin_right = margin
            margin_bottom = margin
            margin_left = margin
        else:
            margin_top = Decimal(margin[0])
            margin_right = Decimal(margin[1])
            margin_bottom = Decimal(margin[2])
            margin_left = Decimal(margin[3])

        if hyphenation is not None:
            hyphenation = _normalize_hyphenation(hyphenation)

        super().__init__(text, respect_newlines_in_text=respect_newlines_in_text, respect_spaces_in_text=respect_spaces_in_text, font=font, font_size=font_size, text_alignment=text_alignment, vertical_alignment=vertical_alignment, horizontal_alignment=horizontal_alignment, font_color=font_color, border_top=border_top, border_right=border_right, border_bottom=border_bottom, border_left=border_left, border_radius_top_left=border_radius_top_left, border_radius_top_right=border_radius_top_right, border_radius_bottom_right=border_radius_bottom_right, border_radius_bottom_left=border_radius_bottom_left, border_color=border_color, border_width=border_width, padding_top=padding_top, padding_right=padding_right, padding_bottom=padding_bottom, padding_left=padding_left, margin_top=margin_top, margin_right=margin_right, margin_bottom=margin_bottom, margin_left=margin_left, fixed_leading=fixed_leading, multiplied_leading=multiplied_leading, background_color=background_color, hyphenation=hyphenation) # TODO - pass the correct params here



def pageSizeFromName(pageSizeName: str) -> Tuple[Decimal, Decimal]:
    pageSizeName = pageSizeName.upper()

    if pageSizeName in 'LETTER' or ('LETTER' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.LETTER_PORTRAIT.value[0], BorbPageSize.LETTER_PORTRAIT.value[1])
    elif 'LETTER' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.LETTER_LANDSCAPE.value[0], BorbPageSize.LETTER_LANDSCAPE.value[1])

    elif pageSizeName in 'LEGAL' or ('LEGAL' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.LEGAL_PORTRAIT.value[0], BorbPageSize.LEGAL_PORTRAIT.value[1])
    elif 'LEGAL' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.LEGAL_LANDSCAPE.value[0], BorbPageSize.LEGAL_LANDSCAPE.value[1])

    elif pageSizeName in 'TABLOID' or ('TABLOID' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.TABLOID_PORTRAIT.value[0], BorbPageSize.TABLOID_PORTRAIT.value[1])
    elif 'TABLOID' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.TABLOID_LANDSCAPE.value[0], BorbPageSize.TABLOID_LANDSCAPE.value[1])

    elif pageSizeName in 'LEDGER' or ('LEDGER' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.LEDGER_PORTRAIT.value[0], BorbPageSize.LEDGER_PORTRAIT.value[1])
    elif 'LEDGER' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.LEDGER_LANDSCAPE.value[0], BorbPageSize.LEDGER_LANDSCAPE.value[1])

    elif pageSizeName in 'EXECUTIVE' or ('EXECUTIVE' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.EXECUTIVE_PORTRAIT.value[0], BorbPageSize.EXECUTIVE_PORTRAIT.value[1])
    elif 'EXECUTIVE' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.EXECUTIVE_LANDSCAPE.value[0], BorbPageSize.EXECUTIVE_LANDSCAPE.value[1])

    elif pageSizeName in 'A0' or ('A0' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A0_PORTRAIT.value[0], BorbPageSize.A0_PORTRAIT.value[1])
    elif 'A0' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A0_LANDSCAPE.value[0], BorbPageSize.A0_LANDSCAPE.value[1])

    elif pageSizeName in 'A1' or ('A1' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A1_PORTRAIT.value[0], BorbPageSize.A1_PORTRAIT.value[1])
    elif 'A1' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A1_LANDSCAPE.value[0], BorbPageSize.A1_LANDSCAPE.value[1])

    elif pageSizeName in 'A2' or ('A2' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A2_PORTRAIT.value[0], BorbPageSize.A2_PORTRAIT.value[1])
    elif 'A2' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A2_LANDSCAPE.value[0], BorbPageSize.A2_LANDSCAPE.value[1])

    elif pageSizeName in 'A3' or ('A3' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A3_PORTRAIT.value[0], BorbPageSize.A3_PORTRAIT.value[1])
    elif 'A3' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A3_LANDSCAPE.value[0], BorbPageSize.A3_LANDSCAPE.value[1])

    elif pageSizeName in 'A4' or ('A4' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A4_PORTRAIT.value[0], BorbPageSize.A4_PORTRAIT.value[1])
    elif 'A4' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A4_LANDSCAPE.value[0], BorbPageSize.A4_LANDSCAPE.value[1])

    elif pageSizeName in 'A5' or ('A5' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A5_PORTRAIT.value[0], BorbPageSize.A5_PORTRAIT.value[1])
    elif 'A5' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A5_LANDSCAPE.value[0], BorbPageSize.A5_LANDSCAPE.value[1])

    elif pageSizeName in 'A6' or ('A6' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A6_PORTRAIT.value[0], BorbPageSize.A6_PORTRAIT.value[1])
    elif 'A6' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A6_LANDSCAPE.value[0], BorbPageSize.A6_LANDSCAPE.value[1])

    elif pageSizeName in 'A7' or ('A7' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A7_PORTRAIT.value[0], BorbPageSize.A7_PORTRAIT.value[1])
    elif 'A7' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A7_LANDSCAPE.value[0], BorbPageSize.A7_LANDSCAPE.value[1])

    elif pageSizeName in 'A8' or ('A8' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A8_PORTRAIT.value[0], BorbPageSize.A8_PORTRAIT.value[1])
    elif 'A8' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A8_LANDSCAPE.value[0], BorbPageSize.A8_LANDSCAPE.value[1])

    elif pageSizeName in 'A9' or ('A9' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A9_PORTRAIT.value[0], BorbPageSize.A9_PORTRAIT.value[1])
    elif 'A9' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A9_LANDSCAPE.value[0], BorbPageSize.A9_LANDSCAPE.value[1])

    elif pageSizeName in 'A10' or ('A10' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.A10_PORTRAIT.value[0], BorbPageSize.A10_PORTRAIT.value[1])
    elif 'A10' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.A10_LANDSCAPE.value[0], BorbPageSize.A10_LANDSCAPE.value[1])

    elif pageSizeName in 'B0' or ('B0' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B0_PORTRAIT.value[0], BorbPageSize.B0_PORTRAIT.value[1])
    elif 'B0' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B0_LANDSCAPE.value[0], BorbPageSize.B0_LANDSCAPE.value[1])

    elif pageSizeName in 'B1' or ('B1' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B1_PORTRAIT.value[0], BorbPageSize.B1_PORTRAIT.value[1])
    elif 'B1' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B1_LANDSCAPE.value[0], BorbPageSize.B1_LANDSCAPE.value[1])

    elif pageSizeName in 'B2' or ('B2' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B2_PORTRAIT.value[0], BorbPageSize.B2_PORTRAIT.value[1])
    elif 'B2' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B2_LANDSCAPE.value[0], BorbPageSize.B2_LANDSCAPE.value[1])

    elif pageSizeName in 'B3' or ('B3' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B3_PORTRAIT.value[0], BorbPageSize.B3_PORTRAIT.value[1])
    elif 'B3' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B3_LANDSCAPE.value[0], BorbPageSize.B3_LANDSCAPE.value[1])

    elif pageSizeName in 'B4' or ('B4' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B4_PORTRAIT.value[0], BorbPageSize.B4_PORTRAIT.value[1])
    elif 'B4' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B4_LANDSCAPE.value[0], BorbPageSize.B4_LANDSCAPE.value[1])

    elif pageSizeName in 'B5' or ('B5' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B5_PORTRAIT.value[0], BorbPageSize.B5_PORTRAIT.value[1])
    elif 'B5' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B5_LANDSCAPE.value[0], BorbPageSize.B5_LANDSCAPE.value[1])

    elif pageSizeName in 'B6' or ('B6' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B6_PORTRAIT.value[0], BorbPageSize.B6_PORTRAIT.value[1])
    elif 'B6' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B6_LANDSCAPE.value[0], BorbPageSize.B6_LANDSCAPE.value[1])

    elif pageSizeName in 'B7' or ('B7' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B7_PORTRAIT.value[0], BorbPageSize.B7_PORTRAIT.value[1])
    elif 'B7' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B7_LANDSCAPE.value[0], BorbPageSize.B7_LANDSCAPE.value[1])

    elif pageSizeName in 'B8' or ('B8' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B8_PORTRAIT.value[0], BorbPageSize.B8_PORTRAIT.value[1])
    elif 'B8' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B8_LANDSCAPE.value[0], BorbPageSize.B8_LANDSCAPE.value[1])

    elif pageSizeName in 'B9' or ('B9' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B9_PORTRAIT.value[0], BorbPageSize.B9_PORTRAIT.value[1])
    elif 'B9' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B9_LANDSCAPE.value[0], BorbPageSize.B9_LANDSCAPE.value[1])

    elif pageSizeName in 'B10' or ('B10' in pageSizeName and 'PORTRAIT' in pageSizeName):
        return (BorbPageSize.B10_PORTRAIT.value[0], BorbPageSize.B10_PORTRAIT.value[1])
    elif 'B10' in pageSizeName and 'LANDSCAPE' in pageSizeName:
        return (BorbPageSize.B10_LANDSCAPE.value[0], BorbPageSize.B10_LANDSCAPE.value[1])

    else:
        raise ValueError('page size must be a valid page size name')



"""Warbler
By Al Sweigart al@inventwithpython.com

TODO_DESCRIPTION"""

__version__ = '0.1.0'

# Importing the Warbler wrappers
from .util import _normalize_text_alignment, _normalize_vertical_alignment, _normalize_horizontal_alignment, _normalize_color, _normalize_hyphenation
from .document import Document
from .page import Page, Paragraph
from .layout import SingleColumnLayout
from .annotation import SquareAnnotation


# Importing directly from Borb:

# Color
from borb.pdf.canvas.color.color import (
    Color,
    RGBColor,
    CMYKColor,
    GrayColor,
    HSVColor,
    HexColor,
    HSVColor,
    X11Color,
)
from borb.pdf.canvas.color.pantone import Pantone

# Image
from borb.pdf.canvas.layout.image.barcode import Barcode
from borb.pdf.canvas.layout.image.barcode import BarcodeType
from borb.pdf.canvas.layout.image.chart import Chart
from borb.pdf.canvas.layout.image.image import Image
from borb.pdf.canvas.layout.layout_element import Alignment

# List
from borb.pdf.canvas.layout.list.list import List
from borb.pdf.canvas.layout.list.ordered_list import OrderedList
from borb.pdf.canvas.layout.list.roman_list import RomanNumeralOrderedList
from borb.pdf.canvas.layout.list.unordered_list import UnorderedList

# PageLayout
from borb.pdf.canvas.layout.page_layout.page_layout import PageLayout
from borb.pdf.canvas.layout.page_layout.multi_column_layout import MultiColumnLayout
#from borb.pdf.canvas.layout.page_layout.multi_column_layout import SingleColumnLayout

#TODO  Doesn't exist for some reason???
from borb.pdf.canvas.layout.page_layout.single_column_layout_with_overflow import (
    SingleColumnLayoutWithOverflow,
)

# Flow
from borb.pdf.canvas.layout.page_layout.inline_flow import InlineFlow
from borb.pdf.canvas.layout.page_layout.block_flow import BlockFlow

# Shape
from borb.pdf.canvas.layout.shape.connected_shape import ConnectedShape
from borb.pdf.canvas.layout.shape.disconnected_shape import DisconnectedShape
from borb.pdf.canvas.layout.smart_art.smart_art import SmartArt

# Table
from borb.pdf.canvas.layout.table.fixed_column_width_table import FixedColumnWidthTable
from borb.pdf.canvas.layout.table.flexible_column_width_table import FlexibleColumnWidthTable
from borb.pdf.canvas.layout.table.table import Table, TableCell
from borb.pdf.canvas.layout.table.table_util import TableUtil
from borb.pdf.canvas.layout.text.heading import Heading

# Forms
from borb.pdf.canvas.layout.forms.form_field import FormField
from borb.pdf.canvas.layout.forms.text_field import TextField
from borb.pdf.canvas.layout.forms.text_area import TextArea
from borb.pdf.canvas.layout.forms.drop_down_list import DropDownList
from borb.pdf.canvas.layout.forms.country_drop_down_list import CountryDropDownList
from borb.pdf.canvas.layout.forms.check_box import CheckBox
from borb.pdf.canvas.layout.forms.push_button import PushButton, JavaScriptPushButton

# Paragraph
#from borb.pdf.canvas.layout.text.paragraph import Paragraph
from borb.pdf.canvas.layout.text.heterogeneous_paragraph import HeterogeneousParagraph
from borb.pdf.canvas.layout.text.heading import Heading
from borb.pdf.canvas.layout.text.chunk_of_text import ChunkOfText
from borb.pdf.canvas.lipsum.lipsum import Lipsum

# Document, Page, PDF
#from borb.pdf.document.document import Document
#from borb.pdf.page.page import Page
from borb.pdf.pdf import PDF

# Additional imports:
from borb.pdf.canvas.font.simple_font.true_type_font import TrueTypeFont



from typing import Union, Tuple
from decimal import Decimal

from borb.pdf.canvas.color.color import Color as BorbColor
from borb.pdf import Alignment as BorbAlignment
from borb.pdf.canvas.geometry.rectangle import Rectangle as BorbRectangle

# Borb tends to force us to use Decimal, so Warbler is more flexible and accepts ints and floats too:
NumberType = Union[Decimal, int, float]

# Borb forces us to use Color objects, so Warbler lets us use strs and RGB tuples:
ColorType = Union[BorbColor, str, Tuple[int, int, int]]

# Borb forces us to use Alignment objects, so Warbler lets us also use strs:
AlignmentType = Union[BorbAlignment, str]

# Borb forces us to use Rectangle objects, so Warbler lets us also use tuples of four numbers:
RectangleType = Union[BorbRectangle, Tuple[NumberType, NumberType, NumberType, NumberType]]

# This is used when you can specify one number to use for four related
# settings, like border for border_top, border_right, border_bottom,
# and border_left:
OneNumForFourType = Union[NumberType, Tuple[NumberType, NumberType, NumberType, NumberType]]
OneBoolForFourType = Union[bool, Tuple[bool, bool, bool, bool]]


from borb.pdf.canvas.layout.annotation.square_annotation import SquareAnnotation as BorbSquareAnnotation
from borb.pdf.canvas.color.color import Color as BorbColor
from borb.pdf.canvas.geometry.rectangle import Rectangle as BorbRectangle

import typing
from typing import Optional, Union, Tuple
from decimal import Decimal

"""
class SquareAnnotation(BorbSquareAnnotation):
    def __init__(
        self,
        bounding_box: Union[Rectangle, Tuple[NumberType, NumberType, NumberType, NumberType]]
        fill_color: Optional[ColorType] = None,
        stroke_color: Optional[ColorType] = None,
        rectangle_difference: Optional[
            Tuple[NumberType, NumberType, NumberType, NumberType]
        ] = None,
    ):
        super().__init__(bounding_box=bounding_box, fill_color=fill_color, stroke_color=stroke_color, rectangle_difference=rectangle_difference)
        """


from borb.pdf import Alignment as BorbAlignment
from borb.pdf.canvas.color.color import Color as BorbColor, HexColor as BorbHexColor, X11Color as BorbX11Color
from borb.pdf.canvas.layout.hyphenation.hyphenation import Hyphenation as BorbHyphenation
from borb.pdf.canvas.font.simple_font.true_type_font import TrueTypeFont as BorbTrueTypeFont

import typing
from typing import Union, Sequence, Tuple, Dict, Optional
from pathlib import Path

from .warblertypes import ColorType

# For our copy of COLOR_DEFINITION, have a lowercase color name
COLOR_DEFINITION: Dict[str, str] = {str(k).lower(): str(v) for k, v in BorbX11Color.COLOR_DEFINITION.items()}



def _normalize_text_alignment(alignment: Union[BorbAlignment, str]) -> BorbAlignment:
    # If alignment is already an enum object, just return it.
    if alignment in (
        BorbAlignment.LEFT,
        BorbAlignment.CENTERED,
        BorbAlignment.RIGHT,
        BorbAlignment.JUSTIFIED,
    ):
        return alignment

    if isinstance(alignment, BorbAlignment) and alignment not in (
        BorbAlignment.LEFT,
        BorbAlignment.CENTERED,
        BorbAlignment.RIGHT,
        BorbAlignment.JUSTIFIED,
    ):
        raise ValueError("text alignment must be 'left', 'center', 'right', or 'justified'")

    # Set alignment based on the string argument
    alignment = alignment.lower()
    if alignment == 'left':
        return BorbAlignment.LEFT
    elif alignment in ('center', 'centered'):
        return BorbAlignment.CENTERED
    elif alignment == 'right':
        return BorbAlignment.RIGHT
    elif alignment == 'justified':
        return BorbAlignment.JUSTIFIED
    else:
        raise ValueError("text alignment must be 'left', 'center', 'right', or 'justified'")

def _normalize_alignment(alignment: Union[BorbAlignment, str]) -> Tuple[BorbAlignment, BorbAlignment]:
    if alignment == BorbAlignment.JUSTIFIED:
        raise ValueError("alignment must include 'top', 'middle', 'bottom', 'left', 'center', or 'right'")

    if alignment in (
        BorbAlignment.TOP,
        BorbAlignment.MIDDLE,
        BorbAlignment.BOTTOM,
    ):
        return (alignment, BorbAlignment.LEFT)

    if alignment in (
        BorbAlignment.LEFT,
        BorbAlignment.CENTERED,
        BorbAlignment.RIGHT,
    ):
        return (BorbAlignment.TOP, alignment)

    vertical_alignment = BorbAlignment.TOP
    horizontal_alignment = BorbAlignment.LEFT

    alignment = alignment.lower()

    if 'top' in alignment:
        vertical_alignment = BorbAlignment.TOP
    elif 'middle' in alignment:
        vertical_alignment = BorbAlignment.MIDDLE
    elif 'bottom' in alignment:
        vertical_alignment = BorbAlignment.BOTTOM

    if 'left' in alignment:
        horizontal_alignment = BorbAlignment.LEFT
    elif 'center' in alignment:
        horizontal_alignment = BorbAlignment.CENTERED
    elif 'right' in alignment:
        horizontal_alignment = BorbAlignment.RIGHT

    return (vertical_alignment, horizontal_alignment)





def _normalize_vertical_alignment(alignment: Union[BorbAlignment, str]) -> BorbAlignment:
    # If alignment is already an enum object, just return it.
    if alignment in (
        BorbAlignment.TOP,
        BorbAlignment.MIDDLE,
        BorbAlignment.BOTTOM,
    ):
        return alignment

    if isinstance(alignment, BorbAlignment) and alignment not in (
        BorbAlignment.TOP,
        BorbAlignment.MIDDLE,
        BorbAlignment.BOTTOM,
    ):
        raise ValueError("vertical alignment must be 'top', 'middle', or 'bottom'")

    # Set alignment based on the string argument
    alignment = alignment.lower()
    if alignment == 'top':
        return BorbAlignment.TOP
    elif alignment == 'middle':
        return BorbAlignment.MIDDLE
    elif alignment == 'bottom':
        return BorbAlignment.BOTTOM
    else:
        raise ValueError("vertical alignment must be 'top', 'middle', or 'bottom'")


def _normalize_horizontal_alignment(alignment: Union[BorbAlignment, str]) -> BorbAlignment:
    # If alignment is already an enum object, just return it.
    if alignment in (
        BorbAlignment.LEFT,
        BorbAlignment.CENTERED,
        BorbAlignment.RIGHT,
    ):
        return alignment

    if isinstance(alignment, BorbAlignment) and alignment not in (
        BorbAlignment.LEFT,
        BorbAlignment.CENTERED,
        BorbAlignment.RIGHT,
    ):
        raise ValueError("horizontal alignment must be 'left', 'centered', or 'right'")

    # Set alignment based on the string argument
    alignment = alignment.lower()
    if alignment == 'left':
        return BorbAlignment.LEFT
    elif alignment in ('center', 'centered'):
        return BorbAlignment.CENTERED
    elif alignment == 'right':
        return BorbAlignment.RIGHT
    else:
        raise ValueError("horizontal alignment must be 'left', 'centered', or 'right'")


def _normalize_color(color: ColorType) -> BorbColor:
    # color that can either be:
    #   - a BorbColor object
    #   - the x11 name string of a color
    #   - the hex string of an RGB color, optionally prefixed with #
    if isinstance(color, BorbColor):
        return color
    elif isinstance(color, str):
        color = color.lower()
        if color in COLOR_DEFINITION:
            return BorbHexColor(COLOR_DEFINITION[color])
        else:
            return BorbHexColor(color)
    else:
        if len(color) != 3:
            raise ValueError('invalid value used for color, must be a color name str or a str of the RGB hex value')
        return BorbHexColor(hex(color[0])[2:].zfill(2) + hex(color[1])[2:].zfill(2) + hex(color[2])[2:].zfill(2))


def _normalize_hyphenation(iso_language_code: Union[BorbHyphenation, str]) -> BorbHyphenation:
    if isinstance(iso_language_code, BorbHyphenation):
        return iso_language_code

    return BorbHyphenation(iso_language_code)


def _normalize_font(fontFile: Union[BorbTrueTypeFont, str, Path]) -> BorbTrueTypeFont:
    if isinstance(fontFile, BorbTrueTypeFont):
        return fontFile

    if isinstance(fontFile, str):
        fontFile = Path(fontFile)

    if isinstance(fontFile, Path):
        return BorbTrueTypeFont.true_type_font_from_file(fontFile)




# TODO - add NormalizeFont that makes it easy to add in custom fonts?


import io
import os
import re
from setuptools import setup, find_packages

scriptFolder = os.path.dirname(os.path.realpath(__file__))
os.chdir(scriptFolder)

# Find version info from module (without importing the module):
with open("src/warbler/__init__.py", "r") as fileObj:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fileObj.read(), re.MULTILINE
    ).group(1)

# Use the README.md content for the long description:
with io.open("README.md", encoding="utf-8") as fileObj:
    long_description = fileObj.read()

setup(
    name="Warbler",
    version=version,
    url="https://github.com/asweigart/warbler",
    author="Al Sweigart",
    author_email="al@inventwithpython.com",
    description=("""TODO_DESCRIPTION"""),
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    test_suite="tests",
    install_requires=["borb==2.1.4"],
    keywords="",
    python_requires=">=3.6",
    classifiers=[
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



#from borb.pdf.canvas.layout.page_layout.page_layout import PageLayout as BorbPageLayout
#from borb.pdf.canvas.layout.page_layout.multi_column_layout import SingleColumnLayout as BorbSingleColumnLayout
#from borb.pdf import PageLayout as BorbPageLayout
from borb.pdf import SingleColumnLayout as BorbSingleColumnLayout
#from borb.pdf import MultiColumnLayout as BorbMultiColumnLayout


class SingleColumnLayout(BorbSingleColumnLayout):
    pass


